import React, { ReactNode, useEffect, useState } from 'react';
import Broker from '@graphpolaris/shared/lib/data-access/socket/broker';
import { useImmer } from 'use-immer';
import CheckCircleOutlineIcon from '@mui/icons-material/CheckCircleOutline';
import { useConfig } from '../store';
import ErrorOutlineIcon from '@mui/icons-material/ErrorOutline';

type Message = {
  message: ReactNode;
  className: string;
};

export const DashboardAlerts = (props: { timer?: number }) => {
  const [messages, setMessages] = useImmer<{ data: any; routingKey: string; message: Message; showing: boolean }[]>([]);
  const timer = props.timer || 1200;
  const config = useConfig();

  function processMessage(message: Message | undefined, data: any, routingKey: string) {
    // Queue the message to be shown for props.time time, then fade it out matching the animation of animate-closemenu fade out time
    if (message) {
      setMessages((draft) => {
        draft.unshift({ data, routingKey, message: message as Message, showing: true });
        return draft;
      });
      setTimeout(() => {
        setMessages((draft) => {
          draft[draft.length - 1].showing = false;
          // draft.pop();
          return draft;
        });

        setTimeout(() => {
          setMessages((draft) => {
            draft.pop();
            return draft;
          });
        }, 300); // matches the animation of animate-closemenu
      }, timer);
    }
  }

  useEffect(() => {
    Broker.instance().subscribeDefault((data: any, routingKey: string) => {
      let message: Message | undefined = undefined;

      // Use the logic below to define which broker messages should be shown as alerts.
      if (routingKey === 'schema_result') {
        message = {
          message: (
            <>
              <CheckCircleOutlineIcon /> Schema graph updated
            </>
          ),
          className: 'alert-success',
        };
      } else if (routingKey === 'query_result') {
        message = {
          message: (
            <>
              <CheckCircleOutlineIcon /> Query Executed!
            </>
          ),
          className: 'alert-success',
        };
      }
      processMessage(message, data, routingKey);
    });
    return () => {
      Broker.instance().unSubscribeDefault();
    };
  }, []);

  useEffect(() => {
    if (config.errors.length > 0) {
      processMessage(
        {
          message: (
            <>
              <ErrorOutlineIcon /> Sever Error
            </>
          ),
          className: 'alert-error',
        },
        undefined,
        'error'
      );
    }
  }, [config.errors]);

  return (
    <>
      {messages &&
        messages.map((m, i) => {
          return (
            <div
              key={i}
              className={
                `absolute bottom-5 right-5 w-fit min-w-[20rem] alert z-50 ` +
                (m.showing ? 'animate-openmenu ' : 'animate-closemenu ') +
                (m.message?.className || '')
              }
              style={{
                transform: `translateY(-${i * 5}rem)`,
              }}
            >
              <span className="flex flex-row content-center gap-3 text-white">{m.message.message}</span>
              {/* {!message && (m.data?.status ? m.data.status : m.routingKey)} */}
            </div>
          );
        })}
    </>
  );
};
