import { PropsWithChildren, useEffect, useRef } from 'react';
import { Dialog, DialogProps } from '../../components/Dialog';
import React from 'react';
import CloseIcon from '@mui/icons-material/Close';
import { useAppDispatch, useQuerybuilderSettings } from '../../data-access';
import { QueryBuilderSettings, setQuerybuilderSettings } from '../../data-access/store/querybuilderSlice';

export const QuerySettingsDialog = React.forwardRef<HTMLDivElement, DialogProps>((props, ref) => {
  const qb = useQuerybuilderSettings();
  const dispatch = useAppDispatch();
  const [state, setState] = React.useState<QueryBuilderSettings>(qb);

  useEffect(() => {
    setState(qb);
  }, [qb, props.open]);

  return (
    <>
      {props.open && (
        <div className="absolute right-20 bottom-5 opacity-100 transition-opacity group-hover:opacity-100 z-50 " ref={ref}>
          <div className="card card-bordered bg-white rounded-none">
            <form
              className="card-body px-0 w-72 py-5"
              onSubmit={(e) => {
                e.preventDefault();
                dispatch(setQuerybuilderSettings(state));
                props.onClose();
              }}
            >
              <div className="card-title p-5 py-0 flex w-full">
                <h2 className="w-full">Query Settings</h2>
                <button className="btn btn-circle btn-sm btn-ghost" onClick={() => props.onClose()}>
                  <CloseIcon fontSize="small" />
                </button>
              </div>
              <div className="divider m-0"></div>
              <div className="form-control px-5">
                <label className="label">
                  <span className="label-text">Limit Value</span>
                </label>
                <input
                  type="number"
                  className="input input-sm input-bordered"
                  placeholder="500"
                  name="limit"
                  value={state.limit}
                  onChange={(e) => setState({ ...state, limit: parseInt(e.target.value) })}
                />
              </div>
              <div className="divider m-0"></div>
              <div className="card-actions mt-1 w-full px-5 flex flex-row">
                <button
                  className="btn btn-secondary flex-grow"
                  onClick={(e) => {
                    e.preventDefault();
                    props.onClose();
                  }}
                >
                  Cancel
                </button>
                <button className="btn btn-primary flex-grow">Apply</button>
              </div>
            </form>
          </div>
        </div>
      )}
    </>
  );
});
